# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: _17_create_hype_cron_job_run_details_table.sql
import dataclasses

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


CREATE_HYPE_CRON_JOB_RUN_DETAILS_TABLE = """-- name: create_hype_cron_job_run_details_table \\:exec
CREATE TABLE IF NOT EXISTS hype_cron_job_run_details (
  jobid          bigint      NOT NULL,
  runid          bigserial   PRIMARY KEY,
  command_type   hype_command_type NOT NULL,
  command_params JSONB       NOT NULL,
  status         hype_cron_job_status_enum,
  schedule_time  timestamptz not null,
  start_time     timestamptz,
  end_time       timestamptz,
  UNIQUE (jobid, schedule_time)
)
"""


@dataclasses.dataclass()
class CreateHypeCronJobRunDetailsTableParams:

    pass

class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def create_hype_cron_job_run_details_table(self, arg: CreateHypeCronJobRunDetailsTableParams) -> None:
        self._conn.execute(sqlalchemy.text(CREATE_HYPE_CRON_JOB_RUN_DETAILS_TABLE))


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def create_hype_cron_job_run_details_table(self, arg: CreateHypeCronJobRunDetailsTableParams) -> None:
        await self._conn.execute(sqlalchemy.text(CREATE_HYPE_CRON_JOB_RUN_DETAILS_TABLE))



# Default queue for enqueued tasks
DEFAULT_QUEUE = "default"
# Default task priority
DEFAULT_PRIORITY = 5
# Default for executors pulling tasks
ANY_QUEUE = "*"
# Default # of executors when running worker via CLI
DEFAULT_EXECUTOR_PROCESSES = 8

WORKER_ADMIN_PROCESS_TIMEOUT = 5.0
WORKER_EXECUTOR_PROCESS_TIMEOUT = 5.0
WORKER_CRON_SCHEDULER_PROCESS_TIMEOUT = 5.0
WORKER_HEARTBEAT_FREQUENCY = 30.0

# How often to refresh queues for round-robin processing
WORKER_EXECUTOR_QUEUE_REFRESH_SECONDS = 300



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: get_cron_jobs_paginated.sql
import dataclasses
import datetime
from typing import Any, AsyncIterator, Iterator, Optional

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


GET_CRON_JOBS_PAGINATED = """-- name: get_cron_jobs_paginated \\:many
SELECT 
    jobid,
    schedule,
    command,
    active,
    jobname,
    job_source,
    activated_at,
    scheduled_jobs_confirmed_until
FROM hyrex_cron_job
ORDER BY jobid DESC
LIMIT :p1 OFFSET :p2
"""


@dataclasses.dataclass()
class GetCronJobsPaginatedParams:
    limit: int
    offset: int


@dataclasses.dataclass()
class GetCronJobsPaginatedRow:
    jobid: int
    schedule: Optional[str]
    command: str
    active: bool
    jobname: str
    job_source: Any
    activated_at: Optional[datetime.datetime]
    scheduled_jobs_confirmed_until: Optional[datetime.datetime]


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def get_cron_jobs_paginated(self, arg: GetCronJobsPaginatedParams) -> Iterator[GetCronJobsPaginatedRow]:
        result = self._conn.execute(sqlalchemy.text(GET_CRON_JOBS_PAGINATED), {"p1": arg.limit, "p2": arg.offset})
        for row in result:
            yield GetCronJobsPaginatedRow(
                jobid=row[0],
                schedule=row[1],
                command=row[2],
                active=row[3],
                jobname=row[4],
                job_source=row[5],
                activated_at=row[6],
                scheduled_jobs_confirmed_until=row[7],
            )


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def get_cron_jobs_paginated(self, arg: GetCronJobsPaginatedParams) -> AsyncIterator[GetCronJobsPaginatedRow]:
        result = await self._conn.stream(sqlalchemy.text(GET_CRON_JOBS_PAGINATED), {"p1": arg.limit, "p2": arg.offset})
        async for row in result:
            yield GetCronJobsPaginatedRow(
                jobid=row[0],
                schedule=row[1],
                command=row[2],
                active=row[3],
                jobname=row[4],
                job_source=row[5],
                activated_at=row[6],
                scheduled_jobs_confirmed_until=row[7],
            )



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: get_task_attempts_by_durable_id.sql
import dataclasses
import datetime
from typing import Any, AsyncIterator, Iterator, Optional
import uuid

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


GET_TASK_ATTEMPTS_BY_DURABLE_ID = """-- name: get_task_attempts_by_durable_id \\:many
SELECT 
    tr.id,
    tr.durable_id,
    tr.root_id,
    tr.parent_id,
    tr.task_name,
    tr.args,
    tr.queue,
    tr.max_retries,
    tr.priority,
    tr.status,
    tr.attempt_number,
    tr.scheduled_start,
    tr.executor_id,
    tr.queued,
    tr.started,
    tr.finished,
    tr.log_link,
    t.source_code,
    res.result
FROM hyrex_task_run tr
LEFT JOIN hyrex_task_def t ON tr.task_name = t.task_name
LEFT JOIN hyrex_task_result res ON tr.id = res.task_id
WHERE tr.durable_id = :p1
ORDER BY tr.attempt_number DESC
"""


@dataclasses.dataclass()
class GetTaskAttemptsByDurableIdParams:
    durable_id: uuid.UUID


@dataclasses.dataclass()
class GetTaskAttemptsByDurableIdRow:
    id: uuid.UUID
    durable_id: uuid.UUID
    root_id: uuid.UUID
    parent_id: Optional[uuid.UUID]
    task_name: str
    args: Any
    queue: str
    max_retries: int
    priority: int
    status: Any
    attempt_number: int
    scheduled_start: Optional[datetime.datetime]
    executor_id: Optional[uuid.UUID]
    queued: Optional[datetime.datetime]
    started: Optional[datetime.datetime]
    finished: Optional[datetime.datetime]
    log_link: Optional[str]
    source_code: Optional[str]
    result: Optional[Any]


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def get_task_attempts_by_durable_id(self, arg: GetTaskAttemptsByDurableIdParams) -> Iterator[GetTaskAttemptsByDurableIdRow]:
        result = self._conn.execute(sqlalchemy.text(GET_TASK_ATTEMPTS_BY_DURABLE_ID), {"p1": arg.durable_id})
        for row in result:
            yield GetTaskAttemptsByDurableIdRow(
                id=row[0],
                durable_id=row[1],
                root_id=row[2],
                parent_id=row[3],
                task_name=row[4],
                args=row[5],
                queue=row[6],
                max_retries=row[7],
                priority=row[8],
                status=row[9],
                attempt_number=row[10],
                scheduled_start=row[11],
                executor_id=row[12],
                queued=row[13],
                started=row[14],
                finished=row[15],
                log_link=row[16],
                source_code=row[17],
                result=row[18],
            )


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def get_task_attempts_by_durable_id(self, arg: GetTaskAttemptsByDurableIdParams) -> AsyncIterator[GetTaskAttemptsByDurableIdRow]:
        result = await self._conn.stream(sqlalchemy.text(GET_TASK_ATTEMPTS_BY_DURABLE_ID), {"p1": arg.durable_id})
        async for row in result:
            yield GetTaskAttemptsByDurableIdRow(
                id=row[0],
                durable_id=row[1],
                root_id=row[2],
                parent_id=row[3],
                task_name=row[4],
                args=row[5],
                queue=row[6],
                max_retries=row[7],
                priority=row[8],
                status=row[9],
                attempt_number=row[10],
                scheduled_start=row[11],
                executor_id=row[12],
                queued=row[13],
                started=row[14],
                finished=row[15],
                log_link=row[16],
                source_code=row[17],
                result=row[18],
            )



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: claim_queued_hype_cron_job_runs.sql
import dataclasses
import datetime
from typing import Any, AsyncIterator, Iterator, Optional

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


CLAIM_QUEUED_HYPE_CRON_JOB_RUNS = """-- name: claim_queued_hype_cron_job_runs \\:many
  UPDATE hype_cron_job_run_details
  SET status = 'PROCESSING',
      start_time = NOW()
  WHERE runid IN (
      SELECT runid
      FROM hype_cron_job_run_details
      WHERE status = 'QUEUED'
        AND schedule_time <= NOW()
      ORDER BY schedule_time ASC
      LIMIT :p1
      FOR UPDATE SKIP LOCKED
  )
  RETURNING jobid, runid, command_type\\:\\:text, command_params, status\\:\\:text, schedule_time, start_time, end_time
"""


@dataclasses.dataclass()
class ClaimQueuedHypeCronJobRunsParams:
    limit: int


@dataclasses.dataclass()
class ClaimQueuedHypeCronJobRunsRow:
    jobid: int
    runid: int
    command_type: str
    command_params: Any
    status: str
    schedule_time: datetime.datetime
    start_time: Optional[datetime.datetime]
    end_time: Optional[datetime.datetime]


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def claim_queued_hype_cron_job_runs(self, arg: ClaimQueuedHypeCronJobRunsParams) -> Iterator[ClaimQueuedHypeCronJobRunsRow]:
        result = self._conn.execute(sqlalchemy.text(CLAIM_QUEUED_HYPE_CRON_JOB_RUNS), {"p1": arg.limit})
        for row in result:
            yield ClaimQueuedHypeCronJobRunsRow(
                jobid=row[0],
                runid=row[1],
                command_type=row[2],
                command_params=row[3],
                status=row[4],
                schedule_time=row[5],
                start_time=row[6],
                end_time=row[7],
            )


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def claim_queued_hype_cron_job_runs(self, arg: ClaimQueuedHypeCronJobRunsParams) -> AsyncIterator[ClaimQueuedHypeCronJobRunsRow]:
        result = await self._conn.stream(sqlalchemy.text(CLAIM_QUEUED_HYPE_CRON_JOB_RUNS), {"p1": arg.limit})
        async for row in result:
            yield ClaimQueuedHypeCronJobRunsRow(
                jobid=row[0],
                runid=row[1],
                command_type=row[2],
                command_params=row[3],
                status=row[4],
                schedule_time=row[5],
                start_time=row[6],
                end_time=row[7],
            )



CREATE_HISTORICAL_TASK_STATUS_COUNTS = """
    CREATE TABLE IF NOT EXISTS hyrex_stats_task_status_counts
    (
        timepoint     TIMESTAMP WITH TIME ZONE PRIMARY KEY,
        queued        INTEGER,
        running       INTEGER,
        waiting       INTEGER,
        failed        INTEGER,
        success       INTEGER,
        lost          INTEGER,
        total         INTEGER,
        queued_delta  INTEGER,
        success_delta INTEGER,
        failed_delta  INTEGER,
        lost_delta    INTEGER
    );

    CREATE INDEX IF NOT EXISTS idx_hstsc_timepoint
        ON hyrex_stats_task_status_counts(timepoint);
"""

FILL_HISTORICAL_TASK_STATUS_COUNTS_TABLE = """
    WITH RECURSIVE timepoints AS (
        -- 1) Start from the larger of:
        --    - The last known timepoint from the stats table (if any)
        --    - 10 minutes ago (rounded to a 15s boundary)
        SELECT GREATEST(
                       COALESCE(
                               (SELECT MAX(timepoint) FROM hyrex_stats_task_status_counts),
                               date_bin(
                                       INTERVAL '15 seconds',
                                       now() - INTERVAL '10 minutes',
                                       TIMESTAMP '2000-01-01 00:00:00+00'
                               )
                       ),
                       date_bin(
                               INTERVAL '15 seconds',
                               now() - INTERVAL '10 minutes',
                               TIMESTAMP '2000-01-01 00:00:00+00'
                       )
               ) + INTERVAL '15 seconds' AS timepoint

        UNION ALL

        -- 2) Keep adding 15 seconds, up to 'now' (also rounded to a 15s boundary)
        SELECT timepoint + INTERVAL '15 seconds'
        FROM timepoints
        WHERE timepoint < date_bin(
                INTERVAL '15 seconds',
                now(),
                TIMESTAMP '2000-01-01 00:00:00+00'
                          )
    ),
                   queue_counts AS (
                       SELECT
                           t.timepoint,
                           COUNT(
                                   CASE
                                       WHEN htr.queued <= t.timepoint
                                           AND (htr.started IS NULL OR htr.started > t.timepoint)
                                           THEN 1
                                       END
                           ) AS queued,
                           COUNT(
                                   CASE
                                       WHEN htr.started <= t.timepoint
                                           AND (htr.finished IS NULL OR htr.finished > t.timepoint)
                                           AND htr.status = 'running'
                                           THEN 1
                                       END
                           ) AS running,
                           COUNT(
                                   CASE
                                       WHEN htr.status = 'waiting'
                                           AND htr.queued <= t.timepoint
                                           AND (htr.finished IS NULL OR htr.finished > t.timepoint)
                                           THEN 1
                                       END
                           ) AS waiting,
                           COUNT(
                                   CASE
                                       WHEN htr.status IN ('failed')
                                           AND htr.finished <= t.timepoint
                                           THEN 1
                                       END
                           ) AS failed,
                           COUNT(
                                   CASE
                                       WHEN htr.status = 'success'
                                           AND htr.finished <= t.timepoint
                                           THEN 1
                                       END
                           ) AS success,
                           COUNT(
                                   CASE
                                       WHEN htr.status = 'lost'
                                           AND htr.finished <= t.timepoint
                                           THEN 1
                                       END
                           ) AS lost
                       FROM timepoints t
                                LEFT JOIN hyrex_task_run htr
                                          ON (
                                              -- Include tasks that existed during this timepoint
                                              htr.queued <= t.timepoint
                                                  AND (
                                                  -- Either they're still in the system
                                                  htr.finished IS NULL
                                                      OR
                                                      -- Or they finished after this timepoint
                                                  htr.finished > t.timepoint
                                                      OR
                                                      -- Or they failed/succeeded/lost at this exact timepoint
                                                  (
                                                      htr.status IN ('failed','success','lost')
                                                          AND htr.finished <= t.timepoint
                                                      )
                                                  )
                                              )
                       GROUP BY t.timepoint
                   ),
                   final_counts AS (
                       SELECT
                           timepoint,
                           queued,
                           running,
                           waiting,
                           failed,
                           success,
                           lost,
                           (queued + running + waiting + failed + lost) AS total,
                           (queued - LAG(queued, 1) OVER (ORDER BY timepoint))   AS queued_delta,
                           (success - LAG(success, 1) OVER (ORDER BY timepoint)) AS success_delta,
                           (failed - LAG(failed, 1) OVER (ORDER BY timepoint))   AS failed_delta,
                           (lost - LAG(lost, 1) OVER (ORDER BY timepoint))       AS lost_delta
                       FROM queue_counts
                   )
    INSERT INTO hyrex_stats_task_status_counts
    SELECT
        timepoint,
        queued,
        running,
        waiting,
        failed,
        success,
        lost,
        total,
        queued_delta,
        success_delta,
        failed_delta,
        lost_delta
    FROM final_counts
    WHERE queued_delta IS NOT NULL
      AND success_delta IS NOT NULL
      AND failed_delta IS NOT NULL
      AND lost_delta IS NOT NULL
    ON CONFLICT (timepoint) DO NOTHING;
"""



QUEUE_WAITING_TASKS = """
UPDATE hyrex_task_run
SET 
    status = 'queued'::task_run_status,
    queued = NOW()
WHERE 
    status = 'waiting'::task_run_status
    AND scheduled_start < NOW();
"""

SET_ORPHANED_TASK_EXECUTION_TO_LOST_AND_RETRY = """
    WITH lost_tasks AS (
        UPDATE hyrex_task_run
            SET status = 'lost'
            WHERE status = 'running'
                AND (
                      executor_id IS NULL
                          OR NOT EXISTS (
                          SELECT 1
                          FROM hyrex_executor
                          WHERE hyrex_executor.id = hyrex_task_run.executor_id
                            AND hyrex_executor.status = 'RUNNING'
                      )
                      )
            RETURNING *
    )
    INSERT INTO hyrex_task_run (
        id,
        durable_id,
        root_id,
        parent_id,
        workflow_run_id,
        workflow_dependencies,
        task_name,
        args,
        queue,
        max_retries,
        priority,
        timeout_seconds,
        status,
        attempt_number,
        idempotency_key,
        queued
    )
    SELECT
        gen_random_uuid(),
        durable_id,
        root_id,
        parent_id,
        workflow_run_id,
        workflow_dependencies,
        task_name,
        args,
        queue,
        max_retries,
        priority,
        timeout_seconds,
        'queued',
        attempt_number + 1,
        idempotency_key,
        NOW()
    FROM lost_tasks
    WHERE attempt_number < max_retries;
"""

SET_EXECUTOR_TO_LOST_IF_NO_HEARTBEAT = """
    WITH lost_executors AS (
        UPDATE hyrex_executor
            SET status = 'LOST'
            WHERE status = 'RUNNING'
                AND (
                      last_heartbeat IS NULL
                          OR last_heartbeat < (NOW() - INTERVAL '5 minutes')
                      )
            RETURNING id, last_heartbeat
    )
    INSERT INTO hyrex_system_logs (
        id,
        timestamp,
        event_name,
        event_body
    )
    SELECT
        gen_random_uuid(),
        NOW(),
        'EXECUTOR_LOST',
        json_build_object(
                'executor_id', id,
                'last_heartbeat', last_heartbeat,
                'reason', CASE
                              WHEN last_heartbeat IS NULL THEN 'No heartbeat recorded'
                              ELSE 'Heartbeat timeout exceeded 5 minutes'
                    END
        )
    FROM lost_executors;
"""



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: get_workflow_run_task_runs.sql
import dataclasses
from typing import Any, AsyncIterator, Iterator, List, Optional
import uuid

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


GET_WORKFLOW_RUN_TASK_RUNS = """-- name: get_workflow_run_task_runs \\:many
WITH latest_attempts AS (
    SELECT 
        id as task_id,
        durable_id,
        task_name,
        status,
        workflow_dependencies,
        queued,
        ROW_NUMBER() OVER (PARTITION BY durable_id ORDER BY queued DESC) as rn
    FROM hyrex_task_run
    WHERE workflow_run_id = :p1
)
SELECT 
    task_id,
    durable_id,
    task_name,
    status,
    workflow_dependencies
FROM latest_attempts
WHERE rn = 1
ORDER BY queued
"""


@dataclasses.dataclass()
class GetWorkflowRunTaskRunsParams:
    workflow_run_id: Optional[uuid.UUID]


@dataclasses.dataclass()
class GetWorkflowRunTaskRunsRow:
    task_id: uuid.UUID
    durable_id: uuid.UUID
    task_name: str
    status: Any
    workflow_dependencies: Optional[List[uuid.UUID]]


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def get_workflow_run_task_runs(self, arg: GetWorkflowRunTaskRunsParams) -> Iterator[GetWorkflowRunTaskRunsRow]:
        result = self._conn.execute(sqlalchemy.text(GET_WORKFLOW_RUN_TASK_RUNS), {"p1": arg.workflow_run_id})
        for row in result:
            yield GetWorkflowRunTaskRunsRow(
                task_id=row[0],
                durable_id=row[1],
                task_name=row[2],
                status=row[3],
                workflow_dependencies=row[4],
            )


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def get_workflow_run_task_runs(self, arg: GetWorkflowRunTaskRunsParams) -> AsyncIterator[GetWorkflowRunTaskRunsRow]:
        result = await self._conn.stream(sqlalchemy.text(GET_WORKFLOW_RUN_TASK_RUNS), {"p1": arg.workflow_run_id})
        async for row in result:
            yield GetWorkflowRunTaskRunsRow(
                task_id=row[0],
                durable_id=row[1],
                task_name=row[2],
                status=row[3],
                workflow_dependencies=row[4],
            )



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: create_cron_job_status_enum.sql
import dataclasses

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


CREATE_CRON_JOB_STATUS_ENUM = """-- name: create_cron_job_status_enum \\:exec
DO $$
BEGIN
    IF NOT EXISTS (SELECT 1 FROM pg_type WHERE typname = 'cron_job_status_enum' AND typnamespace = 'public'\\:\\:regnamespace) THEN
        CREATE TYPE public.cron_job_status_enum AS ENUM (
            'SUCCESS',
            'QUEUED',
            'FAILED'
        );
    END IF;
END $$
"""


@dataclasses.dataclass()
class CreateCronJobStatusEnumParams:

    pass

class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def create_cron_job_status_enum(self, arg: CreateCronJobStatusEnumParams) -> None:
        self._conn.execute(sqlalchemy.text(CREATE_CRON_JOB_STATUS_ENUM))


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def create_cron_job_status_enum(self, arg: CreateCronJobStatusEnumParams) -> None:
        await self._conn.execute(sqlalchemy.text(CREATE_CRON_JOB_STATUS_ENUM))



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: insert_single_task_status_counts_row.sql
import dataclasses
import datetime

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


INSERT_SINGLE_TASK_STATUS_COUNTS_ROW = """-- name: insert_single_task_status_counts_row \\:exec
WITH new_values AS (
    -- Parameters passed from your Go application
    SELECT
        :p1\\:\\:TIMESTAMPTZ AS current_timepoint,
        :p2\\:\\:INTEGER AS current_queued,
        :p3\\:\\:INTEGER AS current_running,
        :p4\\:\\:INTEGER AS current_waiting,
        :p5\\:\\:INTEGER AS current_failed,
        :p6\\:\\:INTEGER AS current_success,
        :p7\\:\\:INTEGER AS current_lost
),
previous_latest AS (
    -- Get the most recent record from the table for relevant delta calculations
    SELECT
        queued AS prev_queued,
        success AS prev_success,
        failed AS prev_failed,
        lost AS prev_lost
    FROM hyrex_stats_task_status_counts
    ORDER BY timepoint DESC
    LIMIT 1
)
INSERT INTO hyrex_stats_task_status_counts (
    timepoint,
    queued,
    running,
    waiting,
    failed,
    success,
    lost,
    total,
    queued_delta,
    success_delta,
    failed_delta,
    lost_delta
)
SELECT
    nv.current_timepoint,
    nv.current_queued,
    nv.current_running,
    nv.current_waiting,
    nv.current_failed,
    nv.current_success,
    nv.current_lost,
    -- Calculate total directly from new values
    (nv.current_queued + nv.current_running + nv.current_waiting + nv.current_failed + nv.current_success + nv.current_lost) AS total_count,

    -- Calculate absolute deltas from the previous row
    -- If no previous row, prev_xxx will be NULL, (current - NULL) is NULL. COALESCE makes it 0.
    COALESCE(nv.current_queued - pl.prev_queued, 0) AS queued_delta,
    COALESCE(nv.current_success - pl.prev_success, 0) AS success_delta,
    COALESCE(nv.current_failed - pl.prev_failed, 0) AS failed_delta,
    COALESCE(nv.current_lost - pl.prev_lost, 0) AS lost_delta
FROM
    new_values nv
LEFT JOIN
    previous_latest pl ON TRUE -- Ensures new_values always processed; pl columns will be NULL if table is empty
ON CONFLICT (timepoint) DO UPDATE SET
    queued = EXCLUDED.queued,
    running = EXCLUDED.running,
    waiting = EXCLUDED.waiting,
    failed = EXCLUDED.failed,
    success = EXCLUDED.success,
    lost = EXCLUDED.lost,
    total = EXCLUDED.total,
    queued_delta = EXCLUDED.queued_delta,
    success_delta = EXCLUDED.success_delta,
    failed_delta = EXCLUDED.failed_delta,
    lost_delta = EXCLUDED.lost_delta
"""


@dataclasses.dataclass()
class InsertSingleTaskStatusCountsRowParams:
    timepoint: datetime.datetime
    queued: int
    running: int
    waiting: int
    failed: int
    success: int
    lost: int


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def insert_single_task_status_counts_row(self, arg: InsertSingleTaskStatusCountsRowParams) -> None:
        self._conn.execute(sqlalchemy.text(INSERT_SINGLE_TASK_STATUS_COUNTS_ROW), {
            "p1": arg.timepoint,
            "p2": arg.queued,
            "p3": arg.running,
            "p4": arg.waiting,
            "p5": arg.failed,
            "p6": arg.success,
            "p7": arg.lost,
        })


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def insert_single_task_status_counts_row(self, arg: InsertSingleTaskStatusCountsRowParams) -> None:
        await self._conn.execute(sqlalchemy.text(INSERT_SINGLE_TASK_STATUS_COUNTS_ROW), {
            "p1": arg.timepoint,
            "p2": arg.queued,
            "p3": arg.running,
            "p4": arg.waiting,
            "p5": arg.failed,
            "p6": arg.success,
            "p7": arg.lost,
        })



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: create_hype_command_type_enum.sql
import dataclasses

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


CREATE_HYPE_COMMAND_TYPE_ENUM = """-- name: create_hype_command_type_enum \\:exec
DO $$
BEGIN
    IF NOT EXISTS (SELECT 1 FROM pg_type WHERE typname = 'hype_command_type') THEN
        CREATE TYPE hype_command_type AS ENUM ('TASK', 'WORKFLOW');
    END IF;
END
$$
"""


@dataclasses.dataclass()
class CreateHypeCommandTypeEnumParams:

    pass

class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def create_hype_command_type_enum(self, arg: CreateHypeCommandTypeEnumParams) -> None:
        self._conn.execute(sqlalchemy.text(CREATE_HYPE_COMMAND_TYPE_ENUM))


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def create_hype_command_type_enum(self, arg: CreateHypeCommandTypeEnumParams) -> None:
        await self._conn.execute(sqlalchemy.text(CREATE_HYPE_COMMAND_TYPE_ENUM))



import contextlib
import sys
from functools import lru_cache
from uuid import UUID

import boto3

from hyrex.dispatcher.performance_dispatcher import PerformanceDispatcher


@lru_cache(maxsize=1)
def get_s3_client():
    """Get or create cached S3 client"""
    return boto3.client("s3")


class LogCapture:
    """Simple log capture"""

    def __init__(self):
        self.logs = []

    def write(self, text):
        self.logs.append(text)
        return len(text)

    def flush(self):
        pass

    def getvalue(self):
        return "".join(self.logs)


class TeeIO:
    """Splits output between original stream and capture"""

    def __init__(self, original_stream, capture):
        self.original_stream = original_stream
        self.capture = capture

    def write(self, data):
        self.original_stream.write(data)
        self.capture.write(data)

    def flush(self):
        self.original_stream.flush()


def get_s3_key(task_id: str):
    return f"hyrex-logs/{task_id}.log"


async def _upload_to_s3_async(task_id: str, bucket_name: str, content: str):
    """
    Upload content to S3 asynchronously
    """
    try:
        key = get_s3_key(task_id=task_id)
        s3_client = get_s3_client()
        s3_client.put_object(Bucket=bucket_name, Key=key, Body=content.encode("utf-8"))
    except Exception as e:
        # TODO: Pass this up the chain
        print(f"Failed to upload logs to S3: {e}", file=sys.__stderr__)


@contextlib.asynccontextmanager
async def write_task_logs_to_s3(
    task_id: str,
    bucket_name: str,
    write_to_console: bool = True,
):
    """
    Async context manager for capturing and uploading task logs to S3.

    Args:
        task_id: Unique identifier for the task
        bucket_name: S3 bucket to store logs
        write_to_console: If True, also write output to console

    Yields:
        str: The S3 URL of the uploaded log file
    """
    log_capture = LogCapture()
    original_stdout = sys.stdout
    original_stderr = sys.stderr

    log_url = f"s3://{bucket_name}/{get_s3_key(task_id=task_id)}"

    try:
        if write_to_console:
            sys.stdout = TeeIO(original_stdout, log_capture)
            sys.stderr = TeeIO(original_stderr, log_capture)
        else:
            sys.stdout = log_capture
            sys.stderr = log_capture
        yield log_url
    finally:
        # Restore original streams
        sys.stdout = original_stdout
        sys.stderr = original_stderr

        # Upload logs if we captured anything
        content = log_capture.getvalue()
        await _upload_to_s3_async(task_id, bucket_name, content)


@contextlib.asynccontextmanager
async def write_task_logs_with_dispatcher(
    task_id: UUID,
    dispatcher: PerformanceDispatcher,
    write_to_console: bool = True,
):
    """
    Async context manager for capturing and uploading task logs via dispatcher.

    Args:
        task_id: Unique identifier for the task
        dispatcher: PerformanceDispatcher instance with write_s3_logs method
        write_to_console: If True, also write output to console
    """
    log_capture = LogCapture()
    original_stdout = sys.stdout
    original_stderr = sys.stderr

    try:
        if write_to_console:
            sys.stdout = TeeIO(original_stdout, log_capture)
            sys.stderr = TeeIO(original_stderr, log_capture)
        else:
            sys.stdout = log_capture
            sys.stderr = log_capture
        yield
    finally:
        # Restore original streams
        sys.stdout = original_stdout
        sys.stderr = original_stderr

        # Upload logs via dispatcher
        content = log_capture.getvalue()
        dispatcher.write_s3_logs(task_id, content)



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: skip_waiting_task_for_workflow_run_id.sql
import dataclasses
from typing import Optional
import uuid

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


SKIP_WAITING_TASK_FOR_WORKFLOW_RUN_ID = """-- name: skip_waiting_task_for_workflow_run_id \\:exec
UPDATE hyrex_task_run
SET status   = 'SKIPPED'\\:\\:task_run_status,
    finished = CURRENT_TIMESTAMP
WHERE status = 'AWAIT_DEPS'
  AND workflow_run_id = :p1
"""


@dataclasses.dataclass()
class SkipWaitingTaskForWorkflowRunIdParams:
    workflow_run_id: Optional[uuid.UUID]


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def skip_waiting_task_for_workflow_run_id(self, arg: SkipWaitingTaskForWorkflowRunIdParams) -> None:
        self._conn.execute(sqlalchemy.text(SKIP_WAITING_TASK_FOR_WORKFLOW_RUN_ID), {"p1": arg.workflow_run_id})


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def skip_waiting_task_for_workflow_run_id(self, arg: SkipWaitingTaskForWorkflowRunIdParams) -> None:
        await self._conn.execute(sqlalchemy.text(SKIP_WAITING_TASK_FOR_WORKFLOW_RUN_ID), {"p1": arg.workflow_run_id})



import inspect
import logging
import os
from typing import Callable, overload

from hyrex import constants
from hyrex.configs import ConfigPhase, TaskConfig, WorkflowConfig
from hyrex.dispatcher import Dispatcher, get_dispatcher
from hyrex.env_vars import EnvVars
from hyrex.hyrex_queue import HyrexQueue
from hyrex.task_wrapper import P, R, TaskWrapper
from hyrex.workflow.workflow import HyrexWorkflow
from hyrex.workflow.workflow_builder import WorkflowBuilder


class HyrexRegistry:
    def __init__(
        self,
        queue: str | HyrexQueue = constants.DEFAULT_QUEUE,
        max_retries: int = 0,
        priority: int = constants.DEFAULT_PRIORITY,
    ):
        self.logger = logging.getLogger(__name__)

        if os.getenv(EnvVars.WORKER_PROCESS):
            self.is_worker_process = True
        else:
            self.is_worker_process = False
        self.dispatcher = get_dispatcher()

        self._task_registry: dict[str, TaskWrapper] = {}
        self._queue_registry: dict[str, HyrexQueue] = {}
        self._workflow_registry: dict[str, HyrexWorkflow] = {}

        # Registry-level task config.
        # Decorated tasks will merge their own configs into this.
        task_config = TaskConfig(
            config_phase=ConfigPhase.registry,
            queue=queue,
            max_retries=max_retries,
            priority=priority,
        )
        self.task_config = task_config

    def register_all_with_db(self):
        # Register tasks and workflows with DB
        for task in self._task_registry.values():
            self.dispatcher.register_task_def(
                task_name=task.task_identifier,
                arg_schema=task.get_arg_schema(),
                task_config=task.task_config,
                cron=task.cron,
                source_code=inspect.getsource(task.func),
            )

        for workflow in self._workflow_registry.values():
            self.dispatcher.register_workflow(
                name=workflow.name,
                source_code=workflow.source_code,
                workflow_dag_json=workflow.workflow_builder.to_json(),
                workflow_arg_schema=workflow.workflow_arg_schema,
                default_config=workflow.workflow_config.get_default_config(),
                cron=workflow.cron,
            )

    def register_task_def(self, task_wrapper: TaskWrapper):
        self.logger.debug(f"Registering task def: {task_wrapper.task_identifier}")
        if self._task_registry.get(task_wrapper.task_identifier):
            raise KeyError(
                f"Task {task_wrapper.task_identifier} is already registered. Task names must be unique."
            )
        self._task_registry[task_wrapper.task_identifier] = task_wrapper

        # Register the task wrapper's queue for tracking concurrency.
        queue = task_wrapper.get_queue()
        if isinstance(queue, str):
            self.register_queue(HyrexQueue(name=queue))
        else:
            self.register_queue(queue)

    def register_queue(self, queue: HyrexQueue | str):
        if self._queue_registry.get(queue.name) and not queue.equals(
            self._queue_registry[queue.name]
        ):
            raise KeyError(
                f"Conflicting concurrency limits on queue name: {queue.name}"
            )

        self._queue_registry[queue.name] = queue

    def register_workflow(self, workflow: HyrexWorkflow):
        self.logger.debug(f"Registering workflow: {workflow.name}")
        if self._workflow_registry.get(workflow.name):
            raise KeyError(
                f"Workflow {workflow.name} is already registered. Workflow names must be unique."
            )
        self._workflow_registry[workflow.name] = workflow

        # Register the workflow's queue for tracking concurrency.
        queue = workflow.get_queue()
        if not queue:
            return

        if isinstance(queue, str):
            self.register_queue(HyrexQueue(name=queue))
        else:
            self.register_queue(queue)

    def get_concurrency_limit(self, queue_name: str):
        if self._queue_registry.get(queue_name):
            return self._queue_registry[queue_name].concurrency_limit
        else:
            return 0

    def get_on_error_handler(self, task_name: str) -> Callable | None:
        task_wrapper = self._task_registry[task_name]
        return task_wrapper.on_error

    def get_retry_backoff(self, task_name: str, attempt_number: int) -> int:
        task_wrapper = self._task_registry[task_name]
        return task_wrapper.get_retry_backoff(attempt_number=attempt_number)

    def get_task_wrappers(self) -> list[TaskWrapper]:
        return self._task_registry.values()

    def get_task_names(self) -> list[str]:
        return [task.task_identifier for task in self._task_registry.values()]

    def get_workflows(self) -> list[HyrexWorkflow]:
        return self._workflow_registry.values()

    def get_task(self, task_name: str):
        return self._task_registry[task_name]

    def add_registry(self, registry: "HyrexRegistry"):
        for task_wrapper in registry.get_task_wrappers():
            self.register_task_def(task_wrapper=task_wrapper)
        for workflow in registry.get_workflows():
            self.register_workflow(workflow=workflow)

    @overload
    def task(self, func: Callable[P, R]) -> TaskWrapper[P, R]: ...

    @overload
    def task(
        self,
        func: None = None,
        *,
        queue: str | HyrexQueue = constants.DEFAULT_QUEUE,
        cron: str | None = None,
        max_retries: int = 0,
        timeout_seconds: int | None = None,
        priority: int = constants.DEFAULT_PRIORITY,
        on_error: Callable | None = None,
        retry_backoff: int | Callable[[int], int] | None = None,
    ) -> Callable[[Callable[P, R]], TaskWrapper[P, R]]: ...

    def task(
        self,
        func: Callable[P, R] | None = None,
        *,
        queue: str | HyrexQueue = constants.DEFAULT_QUEUE,
        cron: str | None = None,
        max_retries: int = 0,
        timeout_seconds: int | None = None,
        priority: int = constants.DEFAULT_PRIORITY,
        on_error: Callable | None = None,
        retry_backoff: int | Callable[[int], int] | None = None,
    ) -> TaskWrapper[P, R] | Callable[[Callable[P, R]], TaskWrapper[P, R]]:
        """
        Create task decorator
        """

        def decorator(func: Callable[P, R]) -> TaskWrapper[P, R]:
            task_identifier = func.__name__
            decorated_task_config = TaskConfig(
                config_phase=ConfigPhase.decorator,
                queue=queue,
                max_retries=max_retries,
                timeout_seconds=timeout_seconds,
                priority=priority,
            )

            task_wrapper = TaskWrapper(
                task_identifier=task_identifier,
                func=func,
                cron=cron,
                task_config=self.task_config.merge(decorated_task_config),
                dispatcher=self.dispatcher,
                on_error=on_error,
                retry_backoff=retry_backoff,
            )
            # Register task within this registry
            self.register_task_def(task_wrapper=task_wrapper)
            return task_wrapper

        if func is not None:
            return decorator(func)
        return decorator

    def workflow(
        self,
        queue: str | HyrexQueue = None,
        timeout_seconds: int | None = None,
        priority: int = None,
        cron: str = None,
        workflow_arg_schema=None,
    ):
        """
        A decorator to register a workflow.
        """

        def decorator(func):
            with WorkflowBuilder() as workflow_builder:
                # Build the workflow by calling the function
                func()

                # Use function name as workflow name
                workflow_name = func.__name__

                # Compile config object
                workflow_config = WorkflowConfig(
                    config_phase=ConfigPhase.decorator, queue=queue, priority=priority
                )

                # Create and return a HyrexWorkflow instance
                workflow = HyrexWorkflow(
                    name=workflow_name,
                    workflow_config=workflow_config,
                    workflow_arg_schema=workflow_arg_schema,
                    workflow_builder=workflow_builder,
                    dispatcher=self.dispatcher,
                    source_code=inspect.getsource(func),
                    cron=cron,
                )
                # Register workflow within this registry
                self.register_workflow(workflow)
            return workflow

        return decorator



from datetime import datetime
from uuid import UUID

from pydantic import BaseModel

from hyrex.dispatcher.dispatcher import Dispatcher
from hyrex.dispatcher.dispatcher_provider import get_dispatcher
from hyrex.durable_run import DurableTaskRun


class HyrexWorkflowContext(BaseModel):
    workflow_run_id: UUID
    workflow_args: dict | None = None
    durable_runs: dict[str, DurableTaskRun] = {}


class HyrexContext(BaseModel):
    task_id: UUID
    durable_id: UUID
    root_id: UUID
    parent_id: UUID | None
    task_name: str
    queue: str
    priority: int
    timeout_seconds: int | None
    scheduled_start: datetime | None
    queued: datetime
    started: datetime
    executor_id: UUID
    attempt_number: int
    max_retries: int
    workflow_run_id: UUID | None

    workflow_context: HyrexWorkflowContext | None = None

    def update_workflow_context(self):
        if self.workflow_run_id:
            dispatcher = get_dispatcher()

            # Initialize workflow context if it doesn't exist
            if not self.workflow_context:
                self.workflow_context = HyrexWorkflowContext(
                    workflow_run_id=self.workflow_run_id
                )

            # Get workflow args
            self.workflow_context.workflow_args = dispatcher.get_workflow_run_args(
                self.workflow_run_id
            )

            # Get workflow durable runs
            durable_ids = dispatcher.get_workflow_durable_runs(self.workflow_run_id)

            # Create dictionary of DurableTaskRun objects indexed by task name
            self.workflow_context.durable_runs = {}

            for durable_id in durable_ids:
                # Create a DurableTaskRun for each durable ID
                durable_run = DurableTaskRun(
                    task_name="",  # Will be updated after refresh
                    durable_id=durable_id,
                )

                # Refresh to get the task runs from database
                durable_run.refresh()

                # Add to the dictionary using task name as the key if we have a task name
                if durable_run.task_name:
                    # TODO: Check for clobbering
                    self.workflow_context.durable_runs[durable_run.task_name] = (
                        durable_run
                    )


# Simple global context
_current_context: HyrexContext | None = None


def get_hyrex_context() -> HyrexContext | None:
    """Get the current Hyrex context."""
    return _current_context


def get_hyrex_workflow_context() -> HyrexWorkflowContext | None:
    """Get the workflow context for the current task execution."""
    if _current_context:
        return _current_context.workflow_context
    return None


def set_hyrex_context(context: HyrexContext) -> None:
    """Set the current Hyrex context."""
    global _current_context
    _current_context = context
    if _current_context.workflow_run_id:
        _current_context.update_workflow_context()


def clear_hyrex_context() -> None:
    """Clear the current Hyrex context."""
    global _current_context
    _current_context = None



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: kv_delete_value.sql
import dataclasses

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


DELETE_VALUE = """-- name: delete_value \\:exec
DELETE FROM hyrex_kv WHERE key = :p1
"""


@dataclasses.dataclass()
class DeleteValueParams:
    key: str


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def delete_value(self, arg: DeleteValueParams) -> None:
        self._conn.execute(sqlalchemy.text(DELETE_VALUE), {"p1": arg.key})


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def delete_value(self, arg: DeleteValueParams) -> None:
        await self._conn.execute(sqlalchemy.text(DELETE_VALUE), {"p1": arg.key})



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: update_hype_cron_job_confirmed_until.sql
import dataclasses
import datetime
from typing import Optional

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


UPDATE_HYPE_CRON_JOB_CONFIRMED_UNTIL = """-- name: update_hype_cron_job_confirmed_until \\:exec
UPDATE hype_cron_job
SET scheduled_jobs_confirmed_until = :p2
WHERE jobid = :p1
"""


@dataclasses.dataclass()
class UpdateHypeCronJobConfirmedUntilParams:
    jobid: int
    scheduled_jobs_confirmed_until: Optional[datetime.datetime]


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def update_hype_cron_job_confirmed_until(self, arg: UpdateHypeCronJobConfirmedUntilParams) -> None:
        self._conn.execute(sqlalchemy.text(UPDATE_HYPE_CRON_JOB_CONFIRMED_UNTIL), {"p1": arg.jobid, "p2": arg.scheduled_jobs_confirmed_until})


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def update_hype_cron_job_confirmed_until(self, arg: UpdateHypeCronJobConfirmedUntilParams) -> None:
        await self._conn.execute(sqlalchemy.text(UPDATE_HYPE_CRON_JOB_CONFIRMED_UNTIL), {"p1": arg.jobid, "p2": arg.scheduled_jobs_confirmed_until})



import logging
import os
import signal
import threading
import time
from multiprocessing import Event, Process, Queue

from hyrex.dispatcher import get_dispatcher
from hyrex.worker.logging import LogLevel, init_logging
from hyrex.worker.messages.admin_messages import (ExecutorHeartbeatMessage,
                                                  ExecutorStoppedMessage,
                                                  NewExecutorMessage,
                                                  TaskCanceledMessage,
                                                  TaskHeartbeatMessage)
from hyrex.worker.messages.root_messages import CancelTaskMessage
from hyrex.worker.utils import is_process_alive


class WorkerAdmin(Process):
    def __init__(
        self,
        root_message_queue: Queue,
        admin_message_queue: Queue,
        log_level: LogLevel,
    ):
        super().__init__()
        self.logger = logging.getLogger(__name__)
        self.log_level = log_level

        self.current_executors = []

        # Outgoing messages to root process
        self.root_message_queue = root_message_queue
        # Incoming messages
        self.admin_message_queue = admin_message_queue
        self._stop_event = Event()

        # To check if root process is running
        self.parent_pid = os.getpid()

    def _message_listener(self):
        while True:
            # Blocking
            message = self.admin_message_queue.get()

            if message == None:
                break

            self.logger.debug(f"Message received: {message}")

            if isinstance(message, NewExecutorMessage):
                self.current_executors.append(message.executor_id)
            elif isinstance(message, ExecutorStoppedMessage):
                # Mark executor as stopped (if not already)
                self.dispatcher.disconnect_executor(message.executor_id)
                self.dispatcher.mark_running_tasks_lost(message.executor_id)
            elif isinstance(message, TaskCanceledMessage):
                self.dispatcher.task_canceled(message.task_id)
            elif isinstance(message, ExecutorHeartbeatMessage):
                self.dispatcher.executor_heartbeat(
                    message.executor_ids, message.timestamp
                )
            elif isinstance(message, TaskHeartbeatMessage):
                self.dispatcher.task_heartbeat(message.task_ids, message.timestamp)

    def run(self):
        init_logging(self.log_level)

        self.logger.info("Initializing admin.")
        self.dispatcher = get_dispatcher()

        # Ignore signals, let main process manage shutdown.
        signal.signal(signal.SIGTERM, signal.SIG_IGN)
        signal.signal(signal.SIGINT, signal.SIG_IGN)

        self.message_listener_thread = threading.Thread(target=self._message_listener)
        self.message_listener_thread.start()
        self.logger.info("Message listener thread now active...")

        try:
            while not self._stop_event.is_set():
                # Get "up_for_cancel" tasks, send to main process
                tasks_to_cancel = self.dispatcher.get_tasks_up_for_cancel()
                for task_id in tasks_to_cancel:
                    self.root_message_queue.put(CancelTaskMessage(task_id=task_id))

                # TODO: Switch to notifications to avoid this sleep.
                self._stop_event.wait(0.5)

                # Confirm parent is still alive
                if not is_process_alive(self.parent_pid):
                    self.logger.warning(
                        "Root process died unexpectedly. Shutting down."
                    )
                    self._stop_event.set()
        finally:
            self.stop()

    def stop(self):
        try:
            # Stop internal message listener
            self.admin_message_queue.put(None)
            self.message_listener_thread.join()
            if self.message_listener_thread.is_alive():
                self.logger.warning("Message listener thread did not exit cleanly.")
            else:
                self.logger.info("Message listener thread closed successfully.")

        except Exception as e:
            print(f"Error during main process shutdown: {e}")
        self.logger.info("Stopping admin")
        self.dispatcher.stop()



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: set_executor_to_lost_if_no_heartbeat.sql
import dataclasses

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


SET_EXECUTOR_TO_LOST_IF_NO_HEARTBEAT = """-- name: set_executor_to_lost_if_no_heartbeat \\:exec
WITH lost_executors AS (
    UPDATE hyrex_executor
        SET status = 'LOST'\\:\\:executor_status
        WHERE status = 'RUNNING'\\:\\:executor_status
            AND (
                  last_heartbeat IS NULL
                      OR last_heartbeat < (NOW() - INTERVAL '5 minutes')
                  )
        RETURNING id, last_heartbeat
)
INSERT INTO hyrex_system_logs (
    id,
    timestamp,
    event_name,
    event_body
)
SELECT
    gen_random_uuid(),
    NOW(),
    'EXECUTOR_LOST',
    json_build_object(
            'executor_id', id,
            'last_heartbeat', last_heartbeat,
            'reason', CASE
                          WHEN last_heartbeat IS NULL THEN 'No heartbeat recorded'
                          ELSE 'Heartbeat timeout exceeded 5 minutes'
                END
    )
FROM lost_executors
"""


@dataclasses.dataclass()
class SetExecutorToLostIfNoHeartbeatParams:

    pass

class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def set_executor_to_lost_if_no_heartbeat(self, arg: SetExecutorToLostIfNoHeartbeatParams) -> None:
        self._conn.execute(sqlalchemy.text(SET_EXECUTOR_TO_LOST_IF_NO_HEARTBEAT))


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def set_executor_to_lost_if_no_heartbeat(self, arg: SetExecutorToLostIfNoHeartbeatParams) -> None:
        await self._conn.execute(sqlalchemy.text(SET_EXECUTOR_TO_LOST_IF_NO_HEARTBEAT))



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: set_log_link.sql
import dataclasses
from typing import Optional
import uuid

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


SET_LOG_LINK = """-- name: set_log_link \\:exec
UPDATE hyrex_task_run
SET log_link = :p1
WHERE id = :p2
"""


@dataclasses.dataclass()
class SetLogLinkParams:
    log_link: Optional[str]
    id: uuid.UUID


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def set_log_link(self, arg: SetLogLinkParams) -> None:
        self._conn.execute(sqlalchemy.text(SET_LOG_LINK), {"p1": arg.log_link, "p2": arg.id})


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def set_log_link(self, arg: SetLogLinkParams) -> None:
        await self._conn.execute(sqlalchemy.text(SET_LOG_LINK), {"p1": arg.log_link, "p2": arg.id})



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: register_executor.sql
import dataclasses
from typing import List
import uuid

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


REGISTER_EXECUTOR = """-- name: register_executor \\:exec
INSERT INTO hyrex_executor (id,
                            name,
                            queue_pattern,
                            queues,
                            worker_name,
                            started,
                            stopped,
                            last_heartbeat,
                            status)
VALUES (:p1, :p2, :p3, :p4, :p5, CURRENT_TIMESTAMP, null, CURRENT_TIMESTAMP, 'RUNNING'\\:\\:executor_status)
"""


@dataclasses.dataclass()
class RegisterExecutorParams:
    id: uuid.UUID
    name: str
    queue_pattern: str
    queues: List[str]
    worker_name: str


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def register_executor(self, arg: RegisterExecutorParams) -> None:
        self._conn.execute(sqlalchemy.text(REGISTER_EXECUTOR), {
            "p1": arg.id,
            "p2": arg.name,
            "p3": arg.queue_pattern,
            "p4": arg.queues,
            "p5": arg.worker_name,
        })


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def register_executor(self, arg: RegisterExecutorParams) -> None:
        await self._conn.execute(sqlalchemy.text(REGISTER_EXECUTOR), {
            "p1": arg.id,
            "p2": arg.name,
            "p3": arg.queue_pattern,
            "p4": arg.queues,
            "p5": arg.worker_name,
        })



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: _05_create_hyrex_stats_task_status_counts_table.sql
import dataclasses

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


CREATE_STATS_TASK_STATUS_COUNTS_TABLE = """-- name: create_stats_task_status_counts_table \\:exec
CREATE TABLE IF NOT EXISTS hyrex_stats_task_status_counts
(
    timepoint     TIMESTAMP WITH TIME ZONE PRIMARY KEY,
    queued        INTEGER,
    running       INTEGER,
    waiting       INTEGER,
    failed        INTEGER,
    success       INTEGER,
    lost          INTEGER,
    total         INTEGER,
    queued_delta  INTEGER,
    success_delta INTEGER,
    failed_delta  INTEGER,
    lost_delta    INTEGER
)
"""


@dataclasses.dataclass()
class CreateStatsTaskStatusCountsTableParams:

    pass

class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def create_stats_task_status_counts_table(self, arg: CreateStatsTaskStatusCountsTableParams) -> None:
        self._conn.execute(sqlalchemy.text(CREATE_STATS_TASK_STATUS_COUNTS_TABLE))


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def create_stats_task_status_counts_table(self, arg: CreateStatsTaskStatusCountsTableParams) -> None:
        await self._conn.execute(sqlalchemy.text(CREATE_STATS_TASK_STATUS_COUNTS_TABLE))



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: kv_list_keys_paginated.sql
import dataclasses
from typing import AsyncIterator, Iterator

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


LIST_KEYS_PAGINATED = """-- name: list_keys_paginated \\:many
SELECT key, value, created_at 
FROM hyrex_kv 
ORDER BY created_at DESC
LIMIT :p1
OFFSET :p2
"""


@dataclasses.dataclass()
class ListKeysPaginatedParams:
    limit: int
    offset: int


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def list_keys_paginated(self, arg: ListKeysPaginatedParams) -> Iterator[models.HyrexKv]:
        result = self._conn.execute(sqlalchemy.text(LIST_KEYS_PAGINATED), {"p1": arg.limit, "p2": arg.offset})
        for row in result:
            yield models.HyrexKv(
                key=row[0],
                value=row[1],
                created_at=row[2],
            )


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def list_keys_paginated(self, arg: ListKeysPaginatedParams) -> AsyncIterator[models.HyrexKv]:
        result = await self._conn.stream(sqlalchemy.text(LIST_KEYS_PAGINATED), {"p1": arg.limit, "p2": arg.offset})
        async for row in result:
            yield models.HyrexKv(
                key=row[0],
                value=row[1],
                created_at=row[2],
            )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: task.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'task.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\ntask.proto\x12\x17hyrex.performanceserver\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cgoogle/protobuf/struct.proto\"\xa3\x05\n\x07TaskRun\x12\n\n\x02id\x18\x01 \x01(\t\x12\x12\n\ndurable_id\x18\x02 \x01(\t\x12\x0f\n\x07root_id\x18\x03 \x01(\t\x12\x11\n\tparent_id\x18\x04 \x01(\t\x12\x11\n\ttask_name\x18\x05 \x01(\t\x12\x33\n\x06status\x18\x06 \x01(\x0e\x32#.hyrex.performanceserver.TaskStatus\x12\x0c\n\x04\x61rgs\x18\x07 \x01(\x0c\x12\r\n\x05queue\x18\x08 \x01(\t\x12\x33\n\x08priority\x18\t \x01(\x0e\x32!.hyrex.performanceserver.Priority\x12\x1c\n\x0ftimeout_seconds\x18\n \x01(\x05H\x00\x88\x01\x01\x12\x38\n\x0fscheduled_start\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x01\x88\x01\x01\x12*\n\x06queued\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12+\n\x07started\x18\r \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x66inished\x18\x0e \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1c\n\x0fworkflow_run_id\x18\x0f \x01(\tH\x02\x88\x01\x01\x12\x1d\n\x15workflow_dependencies\x18\x10 \x03(\t\x12\x16\n\x0e\x61ttempt_number\x18\x11 \x01(\x05\x12\x13\n\x0bmax_retries\x18\x12 \x01(\x05\x12\x0e\n\x06result\x18\x13 \x01(\t\x12\x10\n\x08log_link\x18\x14 \x01(\t\x12\x13\n\x0b\x65xecutor_id\x18\x15 \x01(\tB\x12\n\x10_timeout_secondsB\x12\n\x10_scheduled_startB\x12\n\x10_workflow_run_id\"\xf6\x02\n\x07TaskDef\x12\x11\n\ttask_name\x18\x01 \x01(\t\x12\x30\n\narg_schema\x18\x02 \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x88\x01\x01\x12\x11\n\x04\x63ron\x18\x03 \x01(\tH\x01\x88\x01\x01\x12\x18\n\x0bsource_code\x18\x04 \x01(\tH\x02\x88\x01\x01\x12\r\n\x05queue\x18\x05 \x01(\t\x12\x33\n\x08priority\x18\x06 \x01(\x0e\x32!.hyrex.performanceserver.Priority\x12\x13\n\x0bmax_retries\x18\x07 \x01(\x05\x12\x1c\n\x0ftimeout_seconds\x18\x08 \x01(\x05H\x03\x88\x01\x01\x12\x35\n\x0clast_updated\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x04\x88\x01\x01\x42\r\n\x0b_arg_schemaB\x07\n\x05_cronB\x0e\n\x0c_source_codeB\x12\n\x10_timeout_secondsB\x0f\n\r_last_updated*n\n\x08Priority\x12\x11\n\rP_UNSPECIFIED\x10\x00\x12\x06\n\x02P1\x10\x01\x12\x06\n\x02P2\x10\x02\x12\x06\n\x02P3\x10\x03\x12\x06\n\x02P4\x10\x04\x12\x06\n\x02P5\x10\x05\x12\x06\n\x02P6\x10\x06\x12\x06\n\x02P7\x10\x07\x12\x06\n\x02P8\x10\x08\x12\x06\n\x02P9\x10\t\x12\x07\n\x03P10\x10\n*\xba\x01\n\nTaskStatus\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x0b\n\x07SUCCESS\x10\x01\x12\n\n\x06\x46\x41ILED\x10\x02\x12\x0b\n\x07RUNNING\x10\x03\x12\n\n\x06QUEUED\x10\x04\x12\x11\n\rUP_FOR_CANCEL\x10\x05\x12\x0c\n\x08\x43\x41NCELED\x10\x06\x12\x08\n\x04LOST\x10\x07\x12\x0b\n\x07STOPPED\x10\x08\x12\x0b\n\x07SKIPPED\x10\t\x12\x0e\n\nAWAIT_DEPS\x10\n\x12\x14\n\x10\x41WAIT_START_TIME\x10\x0b\x42\x08Z\x06proto/b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'task_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\006proto/'
  _globals['_PRIORITY']._serialized_start=1157
  _globals['_PRIORITY']._serialized_end=1267
  _globals['_TASKSTATUS']._serialized_start=1270
  _globals['_TASKSTATUS']._serialized_end=1456
  _globals['_TASKRUN']._serialized_start=103
  _globals['_TASKRUN']._serialized_end=778
  _globals['_TASKDEF']._serialized_start=781
  _globals['_TASKDEF']._serialized_end=1155
# @@protoc_insertion_point(module_scope)



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: create_workflow_run_status_enum.sql
import dataclasses

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


CREATE_WORKFLOW_RUN_STATUS_ENUM = """-- name: create_workflow_run_status_enum \\:exec
DO $$
BEGIN
    IF NOT EXISTS (SELECT 1 
                   FROM pg_type 
                   WHERE typname = 'workflow_run_status' 
                     AND typnamespace = 'public'\\:\\:regnamespace) THEN
        CREATE TYPE public.workflow_run_status AS ENUM (
            'SUCCESS',
            'FAILED',
            'RUNNING',
            'UP_FOR_CANCEL',
            'CANCELED',
            'ASLEEP'
        );
    END IF;
END $$
"""


@dataclasses.dataclass()
class CreateWorkflowRunStatusEnumParams:

    pass

class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def create_workflow_run_status_enum(self, arg: CreateWorkflowRunStatusEnumParams) -> None:
        self._conn.execute(sqlalchemy.text(CREATE_WORKFLOW_RUN_STATUS_ENUM))


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def create_workflow_run_status_enum(self, arg: CreateWorkflowRunStatusEnumParams) -> None:
        await self._conn.execute(sqlalchemy.text(CREATE_WORKFLOW_RUN_STATUS_ENUM))



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: get_tasks_paginated.sql
import dataclasses
import datetime
from typing import AsyncIterator, Iterator, Optional

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


GET_TASKS_PAGINATED = """-- name: get_tasks_paginated \\:many
SELECT 
    ROW_NUMBER() OVER (ORDER BY last_updated DESC) as row_number,
    task_name,
    cron_expr,
    source_code,
    last_updated
FROM hyrex_task_def
ORDER BY last_updated DESC
LIMIT :p1 OFFSET :p2
"""


@dataclasses.dataclass()
class GetTasksPaginatedParams:
    limit: int
    offset: int


@dataclasses.dataclass()
class GetTasksPaginatedRow:
    row_number: int
    task_name: str
    cron_expr: Optional[str]
    source_code: Optional[str]
    last_updated: Optional[datetime.datetime]


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def get_tasks_paginated(self, arg: GetTasksPaginatedParams) -> Iterator[GetTasksPaginatedRow]:
        result = self._conn.execute(sqlalchemy.text(GET_TASKS_PAGINATED), {"p1": arg.limit, "p2": arg.offset})
        for row in result:
            yield GetTasksPaginatedRow(
                row_number=row[0],
                task_name=row[1],
                cron_expr=row[2],
                source_code=row[3],
                last_updated=row[4],
            )


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def get_tasks_paginated(self, arg: GetTasksPaginatedParams) -> AsyncIterator[GetTasksPaginatedRow]:
        result = await self._conn.stream(sqlalchemy.text(GET_TASKS_PAGINATED), {"p1": arg.limit, "p2": arg.offset})
        async for row in result:
            yield GetTasksPaginatedRow(
                row_number=row[0],
                task_name=row[1],
                cron_expr=row[2],
                source_code=row[3],
                last_updated=row[4],
            )



from enum import StrEnum
from typing import Union
from uuid import UUID

from pydantic import BaseModel


class CancelTaskMessage(BaseModel):
    task_id: UUID


class SetExecutorTaskMessage(BaseModel):
    executor_id: UUID
    task_id: UUID | None


class TaskRegistrationComplete(BaseModel):
    pass


class HeartbeatRequestMessage(BaseModel):
    pass



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: trigger_execute_queued_cron_job.sql
import dataclasses
from typing import Optional

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


TRIGGER_EXECUTE_QUEUED_CRON_JOB = """-- name: trigger_execute_queued_cron_job \\:one
SELECT execute_queued_cron_job() AS result
"""


@dataclasses.dataclass()
class TriggerExecuteQueuedCronJobParams:

    pass

class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def trigger_execute_queued_cron_job(self, arg: TriggerExecuteQueuedCronJobParams) -> Optional[str]:
        row = self._conn.execute(sqlalchemy.text(TRIGGER_EXECUTE_QUEUED_CRON_JOB)).first()
        if row is None:
            return None
        return row[0]


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def trigger_execute_queued_cron_job(self, arg: TriggerExecuteQueuedCronJobParams) -> Optional[str]:
        row = (await self._conn.execute(sqlalchemy.text(TRIGGER_EXECUTE_QUEUED_CRON_JOB))).first()
        if row is None:
            return None
        return row[0]



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: create_uuid7_func.sql
import dataclasses

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


CREATE_UUID7_FUNCTION = """-- name: create_uuid7_function \\:exec
CREATE OR REPLACE FUNCTION uuid7() RETURNS uuid
    LANGUAGE plpgsql
    VOLATILE  -- ← must be VOLATILE
AS $$
DECLARE
    ts         bigint;
    hex_ts     text;
    rand_hex   text;
    u          text;
    var_nibble text \\:= to_hex(8 + floor(random()*4))\\:\\:text; -- 8–B → RFC 4122 “10xx” variant
BEGIN
    -- 48-bit millisecond Unix epoch
    ts      \\:= floor(extract(epoch FROM clock_timestamp()) * 1000);
    hex_ts  \\:= lpad(to_hex(ts), 12, '0');

    -- 80 bits of random-ish hex (you’ll use 74 of them, per RFC 9562 §6.9)  [oai_citation\\:0‡rfc-editor.org](https\\://www.rfc-editor.org/rfc/rfc9562.html?utm_source=chatgpt.com)
    rand_hex \\:= substr(md5(random()\\:\\:text), 1, 20);

    -- Assemble UUIDv7\\:  time_hi | time_mid | ver+randA | var+randB | randC
    u \\:=
            substr(hex_ts,1,8)               || '-' ||
            substr(hex_ts,9,4)               || '-' ||
            '7'       || substr(rand_hex,1,3)   || '-' ||
            var_nibble|| substr(rand_hex,4,3)   || '-' ||
            substr(rand_hex,7,12);

    RETURN u\\:\\:uuid;
END;
$$
"""


@dataclasses.dataclass()
class CreateUuid7FunctionParams:

    pass

class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def create_uuid7_function(self, arg: CreateUuid7FunctionParams) -> None:
        self._conn.execute(sqlalchemy.text(CREATE_UUID7_FUNCTION))


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def create_uuid7_function(self, arg: CreateUuid7FunctionParams) -> None:
        await self._conn.execute(sqlalchemy.text(CREATE_UUID7_FUNCTION))



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: get_active_hype_cron_jobs.sql
import dataclasses
import datetime
from typing import Any, AsyncIterator, Iterator, Optional

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


GET_ACTIVE_HYPE_CRON_JOBS = """-- name: get_active_hype_cron_jobs \\:many
  SELECT jobid, schedule, command_type\\:\\:text, command_params, active, jobname, activated_at,
  scheduled_jobs_confirmed_until, should_backfill
  FROM hype_cron_job
  WHERE active = true
  ORDER BY jobid
"""


@dataclasses.dataclass()
class GetActiveHypeCronJobsParams:

    pass

@dataclasses.dataclass()
class GetActiveHypeCronJobsRow:
    jobid: int
    schedule: Optional[str]
    command_type: str
    command_params: Any
    active: bool
    jobname: str
    activated_at: Optional[datetime.datetime]
    scheduled_jobs_confirmed_until: Optional[datetime.datetime]
    should_backfill: Optional[bool]


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def get_active_hype_cron_jobs(self, arg: GetActiveHypeCronJobsParams) -> Iterator[GetActiveHypeCronJobsRow]:
        result = self._conn.execute(sqlalchemy.text(GET_ACTIVE_HYPE_CRON_JOBS))
        for row in result:
            yield GetActiveHypeCronJobsRow(
                jobid=row[0],
                schedule=row[1],
                command_type=row[2],
                command_params=row[3],
                active=row[4],
                jobname=row[5],
                activated_at=row[6],
                scheduled_jobs_confirmed_until=row[7],
                should_backfill=row[8],
            )


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def get_active_hype_cron_jobs(self, arg: GetActiveHypeCronJobsParams) -> AsyncIterator[GetActiveHypeCronJobsRow]:
        result = await self._conn.stream(sqlalchemy.text(GET_ACTIVE_HYPE_CRON_JOBS))
        async for row in result:
            yield GetActiveHypeCronJobsRow(
                jobid=row[0],
                schedule=row[1],
                command_type=row[2],
                command_params=row[3],
                active=row[4],
                jobname=row[5],
                activated_at=row[6],
                scheduled_jobs_confirmed_until=row[7],
                should_backfill=row[8],
            )



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: kv_set_value.sql
import dataclasses

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


SET_VALUE = """-- name: set_value \\:exec
INSERT INTO hyrex_kv (key, value) 
VALUES (:p1, :p2)
ON CONFLICT (key) 
DO UPDATE SET value = EXCLUDED.value, created_at = CURRENT_TIMESTAMP
"""


@dataclasses.dataclass()
class SetValueParams:
    key: str
    value: str


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def set_value(self, arg: SetValueParams) -> None:
        self._conn.execute(sqlalchemy.text(SET_VALUE), {"p1": arg.key, "p2": arg.value})


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def set_value(self, arg: SetValueParams) -> None:
        await self._conn.execute(sqlalchemy.text(SET_VALUE), {"p1": arg.key, "p2": arg.value})



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: schedule_cron_job_runs_json.sql
import dataclasses
from typing import Any, Optional

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


SCHEDULE_CRON_JOB_RUNS_JSON = """-- name: schedule_cron_job_runs_json \\:one
WITH input_data AS (
    SELECT 
        json_array_elements(:p1\\:\\:json) AS run_data
),
converted_runs AS (
    SELECT ARRAY_AGG(
        ROW(
            (run_data->>'jobid')\\:\\:BIGINT,
            run_data->>'command',
            (run_data->>'schedule_time')\\:\\:TIMESTAMPTZ
        )\\:\\:cron_job_run_input
    ) AS runs_array
    FROM input_data
)
SELECT 
    json_build_object(
        'success', success,
        'message', message,
        'inserted_count', inserted_count,
        'runids', runids
    ) AS result
FROM converted_runs, 
LATERAL schedule_cron_job_runs(runs_array)
"""


@dataclasses.dataclass()
class ScheduleCronJobRunsJsonParams:
    runs_json: Any


class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def schedule_cron_job_runs_json(self, arg: ScheduleCronJobRunsJsonParams) -> Optional[Any]:
        row = self._conn.execute(sqlalchemy.text(SCHEDULE_CRON_JOB_RUNS_JSON), {"p1": arg.runs_json}).first()
        if row is None:
            return None
        return row[0]


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def schedule_cron_job_runs_json(self, arg: ScheduleCronJobRunsJsonParams) -> Optional[Any]:
        row = (await self._conn.execute(sqlalchemy.text(SCHEDULE_CRON_JOB_RUNS_JSON), {"p1": arg.runs_json})).first()
        if row is None:
            return None
        return row[0]



# Code generated by sqlc. DO NOT EDIT.
# versions:
#   sqlc v1.29.0
# source: set_orphaned_task_execution_to_lost_and_retry.sql
import dataclasses

import sqlalchemy
import sqlalchemy.ext.asyncio

from . import models


SET_ORPHANED_TASK_EXECUTION_TO_LOST_AND_RETRY = """-- name: set_orphaned_task_execution_to_lost_and_retry \\:exec
WITH lost_tasks AS (
    UPDATE hyrex_task_run
        SET status = 'LOST'\\:\\:task_run_status
        WHERE status = 'RUNNING'\\:\\:task_run_status
            AND (
                  executor_id IS NULL
                      OR NOT EXISTS (
                      SELECT 1
                      FROM hyrex_executor
                      WHERE hyrex_executor.id = hyrex_task_run.executor_id
                        AND hyrex_executor.status = 'RUNNING'\\:\\:executor_status
                  )
                  )
        RETURNING id, durable_id, root_id, parent_id, workflow_run_id, workflow_dependencies, task_name, args, queue, max_retries, priority, timeout_seconds, status, attempt_number, scheduled_start, executor_id, queued, started, finished, last_heartbeat, idempotency_key, log_link
)
INSERT INTO hyrex_task_run (
    id,
    durable_id,
    root_id,
    parent_id,
    workflow_run_id,
    workflow_dependencies,
    task_name,
    args,
    queue,
    max_retries,
    priority,
    timeout_seconds,
    status,
    attempt_number,
    idempotency_key,
    queued
)
SELECT
    gen_random_uuid(),
    durable_id,
    root_id,
    parent_id,
    workflow_run_id,
    workflow_dependencies,
    task_name,
    args,
    queue,
    max_retries,
    priority,
    timeout_seconds,
    'QUEUED'\\:\\:task_run_status,
    attempt_number + 1,
    idempotency_key,
    NOW()
FROM lost_tasks
WHERE attempt_number < max_retries
"""


@dataclasses.dataclass()
class SetOrphanedTaskExecutionToLostAndRetryParams:

    pass

class Querier:
    def __init__(self, conn: sqlalchemy.engine.Connection):
        self._conn = conn

    def set_orphaned_task_execution_to_lost_and_retry(self, arg: SetOrphanedTaskExecutionToLostAndRetryParams) -> None:
        self._conn.execute(sqlalchemy.text(SET_ORPHANED_TASK_EXECUTION_TO_LOST_AND_RETRY))


class AsyncQuerier:
    def __init__(self, conn: sqlalchemy.ext.asyncio.AsyncConnection):
        self._conn = conn

    async def set_orphaned_task_execution_to_lost_and_retry(self, arg: SetOrphanedTaskExecutionToLostAndRetryParams) -> None:
        await self._conn.execute(sqlalchemy.text(SET_ORPHANED_TASK_EXECUTION_TO_LOST_AND_RETRY))
